import json

from pip._vendor.packaging.version import Version
from pip.index import FormatControl
from pip.req import InstallRequirement

from piptools.repositories.base import BaseRepository
from piptools.utils import as_tuple, make_install_requirement

import pytest


def pytest_addoption(parser):
    parser.addoption('--run-pypi-integration-tests', action='store_true')


@pytest.fixture
def FakePypiRepository(tmpdir):
    FakeRepository.source_dir = str(tmpdir)
    return FakeRepository


class FakePackageFinder:
    format_control = FormatControl(set(), set())


class FakeRepository(BaseRepository):  # Heavily inspired by: https://raw.githubusercontent.com/nvie/pip-tools/1.6.5/tests/conftest.py
    finder = FakePackageFinder()
    source_dir = None

    def __init__(self, *args, **kwargs):
        with open('tests/fixtures/fake-index.json', 'r') as f:
            self.index = json.load(f)

        with open('tests/fixtures/fake-editables.json', 'r') as f:
            self.editables = json.load(f)

    def find_best_match(self, ireq, prereleases=False):
        if ireq.editable:
            return ireq

        versions = ireq.specifier.filter(self.index[ireq.req.key], prereleases=prereleases)
        best_version = max(versions, key=Version)
        return make_install_requirement(ireq.req.key, best_version, ireq.extras)

    def get_dependencies(self, ireq):
        if ireq.editable:
            return self.editables[str(ireq.link)]

        name, version, extras = as_tuple(ireq)
        # Store non-extra dependencies under the empty string
        extras = ireq.extras + ('',)
        dependencies = [dep for extra in extras for dep in self.index[name][version][extra]]
        return [InstallRequirement.from_line(dep) for dep in dependencies]



from nexus_uploader.nexus import NexusRepositoryClient


def test_append_egg_hash_if_nexus_url_ok():
    nexus_client = NexusRepositoryClient('http://dummy-nexus.wtf', repo_id='pip', artifact_group='ta/ga/da', auth=('user', 'password'))
    url = 'http://dummy-nexus.wtf/content/repositories/pip/ta/ga/da/bonjour/1.2.3-SNAPSHOT/bonjour-1.2.3-SNAPSHOT-py2.py3-none-any.tar.gz'
    assert nexus_client.append_egg_hash_if_nexus_url(url) == 'http://dummy-nexus.wtf/content/repositories/pip/ta/ga/da/bonjour/1.2.3-SNAPSHOT/bonjour-1.2.3-SNAPSHOT-py2.py3-none-any.tar.gz#egg=bonjour==1.2.3-SNAPSHOT'

def test_append_egg_hash_if_nexus_url_external():
    nexus_client = NexusRepositoryClient('http://dummy-nexus.wtf', repo_id='pip', artifact_group='ta/ga/da', auth=('user', 'password'))
    url = 'http://outside.void/content/repositories/pip/ta/ga/da/bonjour/1.2.3-SNAPSHOT/bonjour-1.2.3-SNAPSHOT-py2.py3-none-any.tar.gz'
    assert nexus_client.append_egg_hash_if_nexus_url(url) == url

def test_append_egg_hash_if_nexus_url_hash_already_there():
    nexus_client = NexusRepositoryClient('http://dummy-nexus.wtf', repo_id='pip', artifact_group='ta/ga/da', auth=('user', 'password'))
    url = 'http://dummy-nexus.wtf/content/repositories/pip/ta/ga/da/bonjour/1.2.3-SNAPSHOT/bonjour-1.2.3-SNAPSHOT-py2.py3-none-any.tar.gz#egg=bonjour==1.2.3-SNAPSHOT'
    assert nexus_client.append_egg_hash_if_nexus_url(url) == url



from nexus_uploader.pip_compile import pip_compile
from nexus_uploader.requirements import read_requirements

from unittest.mock import patch
from .pytest_markers import pypi_integration


@pypi_integration  # only ran with --run-pypi-integration-tests
def test_pipcompile_with_real_repository():
    nexus_requirements = read_requirements(pip_compile(['nexus_uploader'], 'dummy-nexus.wtf', raise_if_called))
    assert len(nexus_requirements) == 6


def test_pipcompile_with_mocked_repository(FakePypiRepository):
    with patch('nexus_uploader.pip_compile.PyPIRepository', FakePypiRepository):
        assert read_requirements(pip_compile(['pytest'], 'dummy-nexus.wtf', raise_if_called)) == \
            [('version-locked', ('py', '1.4.31', 'via pytest')), ('version-locked', ('pytest', '3.0.2', ''))]


def raise_if_called(*args, **kwargs):
    raise NotImplementedError
